#define APPLET wave

/*
  Wave - a sound waveform generator 
  reduced for muLinux by M. Andreoli
  added support for console beep (-c switch)

  argv[1]=freq, argv[2]=duration (1/10s)
  based on Wave: 
	 Copyright (C) 1996 Jeff Tranter (jeff_tranter@pobox.com)

*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
/*#include <math.h> */
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/soundcard.h>
#include <sys/kd.h>
#include "math.h"

/* global variables */
int rate          = 8000;
int frequency;
double duration   = 0;		
int channels      = 1;		
int bits          = 8;		
char *device      = "/dev/audio"; 
int fd;				
unsigned char *buffer = 0;

int use_console=0;

void Play()
{
  int numPoints = rate/frequency;
  int numCycles = (int)(frequency*duration)/10;
  int  i,j;
  buffer = malloc(numPoints*channels*(bits/8));

 /* generate one cycle */

      for (i = 0 ; i < numPoints ; i++) {
        j = (int)(128 *sin(2.0*PI*i/numPoints) + 128);
        if(j<0) j=0; if(j>255) j=255;
        buffer[i] = j;
      }


/* write data */
  for (i = 0 ; (i < numCycles) || duration == 0 ; i++)
    write(fd, buffer, numPoints);

}


int wave_main(int argc, char *argv[])
{
  int arg,status;
  char *opt=argv[1];
  int i=1;
  int tone;
  int rc;

/* parse args */

if ( 
( 
(argc<2)
|| (( opt[0]=='-' ) && (opt[1]=='h' )))
)
{
	fprintf(stderr,"Usage: wave [-h] [-c] frequency duration (1/10s)\n");
	fprintf(stderr,"\t [-c] use console beeper\n");
	exit(0);
}

if ( ( opt[0]=='-' ) && (opt[1]=='c' ))
{
	use_console=1;
	i++;
}

  frequency=atol(argv[i]);
  duration=atof(argv[++i]);


/* open device */

if (use_console)
{
	 fd = open("/dev/console", O_WRONLY);
	 if (fd < 0) fd = 1; /* stdout */

	if (frequency!=0.0) tone=1190000/frequency;
  		else tone=0;
  if ( ioctl(fd,KIOCSOUND,tone) != 0 ) exit(1);

  // (con diet ottenevo 10 volte la durata normale!
  // usleep((long)duration*100000);  
   usleep((long)duration*100000);  

  if( ioctl(fd,KIOCSOUND,0) != 0 ) exit(1);

}
else
{

  fd = open(device, O_WRONLY|O_CREAT|O_TRUNC, 0777);

  if (fd == -1)
	{
     fprintf(stderr, "%s:unable to open\n", device);
	exit(1);
	}
/* init device */
  status = ioctl(fd, SOUND_PCM_WRITE_BITS, &bits);
  if (status == -1)
	{
        fprintf(stderr,"SOUND_PCM_WRITE_BITS ioctl failed\n");
	exit(1);
	}
  rc=ioctl(fd, SOUND_PCM_WRITE_CHANNELS, &channels);
  rc=ioctl(fd, SOUND_PCM_WRITE_RATE, &rate);
  arg = AFMT_U8;
  rc=ioctl(fd, SOUND_PCM_SETFMT, &arg);
  if (rc !=0 ) exit(1);

/* generate and play */
  Play();
}


/* close device */
 if(fd>2) close(fd);
  return 0;
}
